@@ -25,6 +25,12 @@
 # include "libunwind-x86_64.h"
 #elif defined __tilegx__
 # include "libunwind-tilegx.h"
+#elif defined __s390x__
+# include "libunwind-s390x.h"
+#elif defined __riscv || defined __riscv__
+# include "libunwind-riscv.h"
+#elif defined __loongarch64
+# include "libunwind-loongarch64.h"
 #else
 # error "Unsupported arch"
 #endif
